@@ -351,6 +351,9 @@ print.cqp_attr <- function(x, ...) {
  # ------------------------------------------------------------------------
  ##
 corpus <- function(corpus.name) {
+        if (! corpus.name %in% cqi_list_corpora()) {
+	    stop("This corpus does not exist, see cqi_list_corpora()");
+	}
 	x <- 0;
 	class(x) <- "cqp_corpus";
 	attr(x, "cqp_corpus.name") <- corpus.name;
@@ -779,7 +782,7 @@ cqp_flist.cqp_subcorpus <- function(x, anchor, attribute, left.context=0, right.
 		fdist <- cqi_fdist1(qualified.subcorpus.name, anchor, attribute, cutoff=cutoff, offset=offset);
 		id <- fdist[,1];
 		flist <- fdist[,2];
-		names(flist) <- cqi_cpos2str(
+		names(flist) <- cqi_id2str(
 			paste(parent.cqp_corpus.name, attribute, sep="."),
 			id
 		);
