@@ -290,8 +290,7 @@ check_access_conditions(Corpus *corpus, int verbose)
   /* get password data only if we have user / group access restrictions */
   if (corpus->userAccessList || corpus->groupAccessList) {
     /*     pwd = getpwuid(getuid()); */
-    fprintf(
-        stderr,
+    Rprintf(
         "CL Error: Sorry, user/group access restrictions are disabled due to incompatibilities.\n");
     if (pwd == NULL) {
       perror("getpwuid(getuid()): can't get user information");
@@ -327,13 +326,12 @@ check_access_conditions(Corpus *corpus, int verbose)
       for (i = 0; i < nr_groups; i++) {
 
         /*      grpent = getgrgid(gidset[i]); */
-        fprintf(
-            stderr,
+        Rprintf(
             "CL Error: Sorry, user/group access restrictions are disabled due to incompatibilities.\n");
 
         if (grpent == NULL) {
           perror("getgrgid(2): ");
-          fprintf(stderr, "Can't get group information for gid %d\n",
+         Rprintf( "Can't get group information for gid %d\n",
               (int) gidset[i]);
           access_ok = 0;
         }
@@ -357,12 +355,12 @@ check_access_conditions(Corpus *corpus, int verbose)
 #if 0
       IDList l;
 
-      fprintf(stderr,
+     Rprintf(
           "The corpus ``%s'' may be used on the following systems only:\n",
           corpus->id ? corpus->id : "(unknown)");
 
       for (l = corpus->hostAccessList; l; l = l->next) {
-        fprintf(stderr, "\t%s\n", l->string ? l->string : "(null)");
+       Rprintf( "\t%s\n", l->string ? l->string : "(null)");
       }
 #endif
     }
@@ -372,7 +370,7 @@ check_access_conditions(Corpus *corpus, int verbose)
   }
 
   if (!access_ok) {
-    fprintf(stderr, "User ``%s'' is not authorized to access corpus ``%s''\n",
+   Rprintf( "User ``%s'' is not authorized to access corpus ``%s''\n",
         (pwd && pwd->pw_name) ? pwd->pw_name : "(unknown)", corpus->name);
   }
 
@@ -406,7 +404,7 @@ cl_new_corpus(char *registry_dir, char *registry_name)
   /* new function replaces call to cl_string_canonical ... */
   cl_id_tolower(canonical_name);
   if (!cl_id_validate(canonical_name)) {
-    fprintf(stderr, "cl_new_corpus: <%s> is not a valid corpus name\n", registry_name);
+   Rprintf( "cl_new_corpus: <%s> is not a valid corpus name\n", registry_name);
   }
 
   /* ------------------------------------------------------------------ */
@@ -424,7 +422,7 @@ cl_new_corpus(char *registry_dir, char *registry_name)
     cregin = find_corpus_registry(registry_dir, canonical_name, &real_registry_name);
 
     if (cregin == NULL) {
-      fprintf(stderr, "cl_new_corpus: can't locate <%s> in %s\n", registry_name, registry_dir);
+     Rprintf( "cl_new_corpus: can't locate <%s> in %s\n", registry_name, registry_dir);
     }
     else {
       cregrestart(cregin);
@@ -439,8 +437,7 @@ cl_new_corpus(char *registry_dir, char *registry_name)
           loaded_corpora = corpus;
           /* check whether ID field corresponds to name of registry file */
           if (corpus->id && (strcmp(corpus->id, canonical_name) != 0)) {
-            fprintf(
-                stderr,
+            Rprintf(
                 "CL warning: ID field '%s' does not match name of registry file %s/%s\n",
                 corpus->id, real_registry_name, canonical_name);
           }
@@ -551,24 +548,24 @@ describe_corpus(Corpus *corpus)
 
   assert(corpus != NULL);
 
-  printf("\n\n-------------------- CORPUS SETUP ---------------------\n\n");
+ Rprintf("\n\n-------------------- CORPUS SETUP ---------------------\n\n");
 
-  printf("ID:\t%s\n", corpus->id ? corpus->id : "(null)");
-  printf("Name:\t%s\n", corpus->name ? corpus->name : "(null)");
-  printf("Path:\t%s\n", corpus->path ? corpus->path : "(null)");
-  printf("Info:\t%s\n", corpus->info_file ? corpus->info_file : "(null)");
+ Rprintf("ID:\t%s\n", corpus->id ? corpus->id : "(null)");
+ Rprintf("Name:\t%s\n", corpus->name ? corpus->name : "(null)");
+ Rprintf("Path:\t%s\n", corpus->path ? corpus->path : "(null)");
+ Rprintf("Info:\t%s\n", corpus->info_file ? corpus->info_file : "(null)");
 
-  printf("\nRegistry Directory:\t%s\n",
+ Rprintf("\nRegistry Directory:\t%s\n",
       corpus->registry_dir ? corpus->registry_dir : "(null)");
-  printf("Registry Name:     \t%s\n\n",
+ Rprintf("Registry Name:     \t%s\n\n",
       corpus->registry_name ? corpus->registry_name : "(null)");
 
-  printf("Attributes:\n");
+ Rprintf("Attributes:\n");
   for (attr = (Attribute *) (corpus->attributes); attr != NULL; attr
       = (Attribute *) (attr->any.next))
     describe_attribute(attr);
 
-  printf("\n\n------------------------- END -------------------------\n\n");
+ Rprintf("\n\n------------------------- END -------------------------\n\n");
 }
 
 /* ---------------------------------------------------------------------- */
@@ -850,7 +847,7 @@ add_corpus_property(Corpus *corpus, char *property, char *value)
   int i;
 
   if (cl_corpus_property(corpus, property) != NULL) {
-    fprintf(stderr,
+   Rprintf(
         "REGISTRY WARNING (%s/%s): re-defintion of property '%s' (ignored)\n",
         cregin_path, cregin_name, property);
   }
